package javax.swing;

import java.util.Enumeration;
import java.util.Hashtable;
import java.io.IOException;
import java.io.Serializable;
import java.io.ObjectOutputStream;

class ArrayTable implements Cloneable
{
    private Object table;
    private static final int ARRAY_BOUNDARY = 8;
    
    ArrayTable() {
        this.table = null;
    }
    
    static void writeArrayTable(ObjectOutputStream objectOutputStream, ArrayTable arrayTable) throws IOException {
        Object[] keys;
        if (arrayTable == null || (keys = arrayTable.getKeys((Object[])null)) == null) {
            objectOutputStream.writeInt(0);
        }
        else {
            int val = 0;
            for (int i = 0; i < keys.length; ++i) {
                Object o = keys[i];
                if ((o instanceof Serializable && arrayTable.get(o) instanceof Serializable) || (o instanceof ClientPropertyKey && ((ClientPropertyKey)o).getReportValueNotSerializable())) {
                    ++val;
                }
                else {
                    keys[i] = null;
                }
            }
            objectOutputStream.writeInt(val);
            if (val > 0) {
                for (Object obj : keys) {
                    if (obj != null) {
                        objectOutputStream.writeObject(obj);
                        objectOutputStream.writeObject(arrayTable.get(obj));
                        if (--val == 0) {
                            break;
                        }
                    }
                }
            }
        }
    }
    
    public void put(Object o, Object value) {
        if (this.table == null) {
            this.table = new Object[] { o, value };
        }
        else {
            int size = this.size();
            if (size < 8) {
                if (this.containsKey(o)) {
                    Object[] array2 = (Object[])(Object[])this.table;
                    int i = 0;
                    while (i < array2.length - 1) {
                        if (array2[i].equals(o)) {
                            array2[i + 1] = value;
                            break;
                        }
                        i += 2;
                    }
                }
                else {
                    Object[] array = (Object[])(Object[])this.table;
                    int length = array.length;
                    Object[] table = new Object[length + 2];
                    System.arraycopy(array, 0, table, 0, length);
                    table[length] = o;
                    table[length + 1] = value;
                    this.table = table;
                }
            }
            else {
                if (size == 8 && this.isArray()) {
                    this.grow();
                }
                ((Hashtable)this.table).put(o, value);
            }
        }
    }
    
    public Object get(Object o) {
        Object value = null;
        if (this.table != null) {
            if (this.isArray()) {
                Object[] array = (Object[])(Object[])this.table;
                int i = 0;
                while (i < array.length - 1) {
                    if (array[i].equals(o)) {
                        value = array[i + 1];
                        break;
                    }
                    i += 2;
                }
            }
            else {
                value = ((Hashtable)this.table).get(o);
            }
        }
        return value;
    }
    
    public int size() {
        if (this.table == null) {
            return 0;
        }
        int size;
        if (this.isArray()) {
            size = ((Object[])(Object[])this.table).length / 2;
        }
        else {
            size = ((Hashtable)this.table).size();
        }
        return size;
    }
    
    public boolean containsKey(Object o) {
        boolean containsKey = false;
        if (this.table != null) {
            if (this.isArray()) {
                Object[] array = (Object[])(Object[])this.table;
                int i = 0;
                while (i < array.length - 1) {
                    if (array[i].equals(o)) {
                        containsKey = true;
                        break;
                    }
                    i += 2;
                }
            }
            else {
                containsKey = ((Hashtable)this.table).containsKey(o);
            }
        }
        return containsKey;
    }
    
    public Object remove(Object o) {
        Object remove = null;
        if (o == null) {
            return null;
        }
        if (this.table != null) {
            if (this.isArray()) {
                int n = -1;
                Object[] array = (Object[])(Object[])this.table;
                int i = array.length - 2;
                while (i >= 0) {
                    if (array[i].equals(o)) {
                        n = i;
                        remove = array[i + 1];
                        break;
                    }
                    i -= 2;
                }
                if (n != -1) {
                    Object[] array2 = new Object[array.length - 2];
                    System.arraycopy(array, 0, array2, 0, n);
                    if (n < array2.length) {
                        System.arraycopy(array, n + 2, array2, n, array2.length - n);
                    }
                    this.table = ((array2.length == 0) ? null : array2);
                }
            }
            else {
                remove = ((Hashtable)this.table).remove(o);
            }
            if (this.size() == 7 && !this.isArray()) {
                this.shrink();
            }
        }
        return remove;
    }
    
    public void clear() {
        this.table = null;
    }
    
    public Object clone() {
        ArrayTable arrayTable = new ArrayTable();
        if (this.isArray()) {
            Object[] array = (Object[])(Object[])this.table;
            for (int i = 0; i < array.length - 1; i += 2) {
                arrayTable.put(array[i], array[i + 1]);
            }
            return arrayTable;
        }
        Hashtable<Object, Object> hashtable = (Hashtable)this.table;
        Enumeration<Object> keys = hashtable.keys();
        while (keys.hasMoreElements()) {
            Object nextElement = keys.nextElement();
            arrayTable.put(nextElement, hashtable.get(nextElement));
        }
        return arrayTable;
    }
    
    public Object[] getKeys(Object[] array) {
        if (this.table == null) {
            return null;
        }
        if (this.isArray()) {
            Object[] array2 = (Object[])(Object[])this.table;
            if (array == null) {
                array = new Object[array2.length / 2];
            }
            for (int j = 0, n = 0; j < array2.length - 1; j += 2, ++n) {
                array[n] = array2[j];
            }
            return array;
        }
        else {
            Hashtable<Object, Object> hashtable = (Hashtable)this.table;
            Enumeration<Object> keys = hashtable.keys();
            int i = hashtable.size();
            if (array == null) {
                array = new Object[i];
            }
            while (i > 0) {
                array[--i] = keys.nextElement();
            }
            return array;
        }
    }
    
    private boolean isArray() {
        return this.table instanceof Object[];
    }
    
    private void grow() {
        Object[] array = (Object[])(Object[])this.table;
        Hashtable<Object, Object> table = new Hashtable(array.length / 2);
        for (int i = 0; i < array.length; i += 2) {
            table.put(array[i], array[i + 1]);
        }
        this.table = table;
    }
    
    private void shrink() {
        Hashtable<Object, Object> hashtable = (Hashtable)this.table;
        Object[] table = new Object[hashtable.size() * 2];
        Enumeration<Object> keys = hashtable.keys();
        int n = 0;
        while (keys.hasMoreElements()) {
            Object nextElement = keys.nextElement();
            table[n] = nextElement;
            table[n + 1] = hashtable.get(nextElement);
            n += 2;
        }
        this.table = table;
    }
}
